# -*- coding: utf-8 -*-
'''
Example for the usage of the "fastcluster" package.

    http://CRAN.R-project.org/package=fastcluster

Daniel Müllner, muellner@math.stanford.edu
'''
import fastcluster
import numpy as np
import matplotlib.pyplot as plt
from scipy.cluster.hierarchy import dendrogram
from scipy.spatial.distance import pdist

import sys # Provide code for both Python 2 and Python 3
if sys.hexversion < 0x03000000:
    from urllib2 import urlopen
else:
    from urllib.request import urlopen
f = urlopen('http://scipy-cluster.googlecode.com/svn/trunk/hcluster'
            '/tests/iris.txt')
X = np.loadtxt(f)
f.close()
N = len(X)

classes = ['g'] * 50 + ['r'] * 50 + ['c'] * 50

def plot_with_labels(Z, num_clust):
    # plot the dendrogram
    threshold = Z[-num_clust + 1, 2]
    dg = dendrogram(Z, no_labels = True, color_threshold = threshold)
    # plot color bars under the leaves
    color = [classes[k] for k in dg['leaves']]
    b = .1 * Z[-1, 2]
    plt.bar(np.arange(N) * 10, np.ones(N) * b, bottom = -b, width = 10,
            color = color, edgecolor = 'none')
    plt.gca().set_ylim((-b, None))
    plt.show()

Z = fastcluster.linkage(X, method = 'single')
plot_with_labels(Z, 2)

D = pdist(X, metric = 'cityblock')
Z = fastcluster.linkage(D, method = 'weighted')
plot_with_labels(Z, 3)
